﻿using System;
using System.Collections;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Windows.Forms;
using TMXmlRpcLib;
using System.Net;

namespace TMChecker
{
	public partial class FormTMChecker : Form
	{
		XmlRpcClient xmlRpcClient = null;
        int playerCount = 0;

		public FormTMChecker()
		{
			InitializeComponent();

            if (checkBoxStartMinimized.Checked)
            {
                WindowState = FormWindowState.Minimized;
                timerStartMinimized.Enabled = true;
            }

            if (checkBoxAutostart.Checked)
            {
                StartAutoStart();
            }
		}

        private void timerStartMinimized_Tick(object sender, EventArgs e)
        {
            timerStartMinimized.Enabled = false;
            WindowState = FormWindowState.Minimized;
            Hide();
        }

        private int PlayerCount
        {
            get { return playerCount; }
            set
            {
                if (playerCount != value)
                {
                    int oldPlayerCount = playerCount;
                    playerCount = Math.Max(value, 0);
                    if (oldPlayerCount == 0 && playerCount > 0)
                    {
                        notifyIcon.Icon = Properties.Resources.TrackMania16Alert;

						string balloonText = "Player online:";
						foreach (string login in users.Keys)
						{
							balloonText += "\r\n" + login;
						}
						notifyIcon.BalloonTipText = balloonText;
                        notifyIcon.ShowBalloonTip(10);
                    }
                    else if (playerCount == 0 && oldPlayerCount > 0)
                        notifyIcon.Icon = Properties.Resources.TrackMania16;
                }
            }
        }

		private void Connect()
		{
			//proxyTM.Url = "http://localhost:5000";
			////proxyTM.Url = "http://172.25.160.17:5000";

			string serverIPAddress = textBoxHost.Text;
			IPAddress[] addresslist = Dns.GetHostAddresses(serverIPAddress);
			foreach (IPAddress ia in addresslist)
			{
				if(ia.AddressFamily == System.Net.Sockets.AddressFamily.InterNetwork)
				{
					serverIPAddress = ia.ToString();
					break;
				}
			}

			xmlRpcClient = new XmlRpcClient(serverIPAddress, int.Parse(textBoxPort.Text));

            GbxCall call2 = xmlRpcClient.Request("Authenticate", new object[] { textBoxLogin.Text, textBoxPassword.Text });
            if (call2.Error)
            {
                MessageBox.Show(call2.ErrorString + " (" + call2.ErrorCode.ToString() + ")", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }
            if (call2.Params[0] is Boolean && (bool)call2.Params[0] == true)
            {
                xmlRpcClient.EventOnDisconnectCallback += new OnDisconnectHandler(xmlRpcClient_EventOnDisconnectCallback);
                xmlRpcClient.EventGbxCallback += new GbxCallbackHandler(xmlRpcClient_EventGbxCallback);
                xmlRpcClient.EnableCallbacks(true);

                //GbxCall call3 = xmlRpcClient.Request("GetMaxPlayers", new object[] { });
                GbxCall call4 = xmlRpcClient.Request("GetCurrentRanking", new object[] { 200, 0 });
                //GbxCall call5 = xmlRpcClient.Request("GetDetailedPlayerInfo", new object[] { "dgis/155.132.39.19:2350" });

                //"Score"]	0	
                //"NbrLapsFinished"]	0	
                //"PlayerId"]	21	
                //"Rank"]	1	
                //"NickName"]	"dgis"	
                //"Login"]	"dgis/155.132.39.19:2350"	
                //"BestCheckpoints"]	{System.Collections.ArrayList}	
                //"BestTime"]	-1	
                //"LadderScore"]	0.0	

                listViewUsers.Enabled = true;
                listViewUsers.Items.Clear();

                ArrayList userList = call4.Params[0] as ArrayList;
                foreach (Hashtable userHashtable in userList)
                {
                    string userLogin = userHashtable["Login"] as string;
                    users[userLogin] = userHashtable;

					string userNickName = userHashtable["NickName"] as string;
                    ListViewItem lvi = new ListViewItem(userNickName);
                    lvi.SubItems.Add(userLogin);
                    lvi.SubItems.Add(((int)userHashtable["PlayerId"]).ToString());
                    lvi.Tag = userLogin;
                    listViewUsers.Items.Add(lvi);

                    userHashtable["LVI"] = lvi;

					textBoxLog.AppendText(DateTime.Now.ToString("yyyy/MM/dd HH:mm:ss.fff") + " '" + userNickName + "' is already connected.\r\n");
                }

                PlayerCount = userList.Count;

                buttonStart.Enabled = false;
                buttonStop.Enabled = true;

                notifyIcon.Icon = Properties.Resources.TrackMania16;

				textBoxLog.AppendText(DateTime.Now.ToString("yyyy/MM/dd HH:mm:ss.fff") + " Connected to the server.\r\n");
            }

		}

		private Hashtable users = new Hashtable();

		private void Disconnect()
		{
			textBoxLog.AppendText(DateTime.Now.ToString("yyyy/MM/dd HH:mm:ss.fff") + " Disconnected from the server.\r\n");

            notifyIcon.Icon = Properties.Resources.TrackMania16Grey;
            listViewUsers.Items.Clear();
			listViewUsers.Enabled = false;
            if (xmlRpcClient != null)
            {
                try
                {
                    xmlRpcClient.Dispose();
                }
                catch
                {
                }
            }
			xmlRpcClient = null;
			buttonStart.Enabled = true;
			buttonStop.Enabled = false;
		}

		void xmlRpcClient_EventGbxCallback(object o, GbxCallbackEventArgs e)
		{
			if(InvokeRequired) // Go into the GUI thread
			{
                //Trace.WriteLine(e.Response.Xml.ToString());

                Invoke(new GbxCallbackHandler(xmlRpcClient_EventGbxCallback), new object[] { o, e });
                return;
			}

			switch (e.Response.MethodName)
			{
				case "TrackMania.PlayerConnect":
					{
						string userLogin = e.Response.Params[0] as string;
						Hashtable userHashtable = new Hashtable();
						users[userLogin] = userHashtable;

						ListViewItem lvi = new ListViewItem("");
						lvi.SubItems.Add(userLogin);
						lvi.SubItems.Add("");
						lvi.Tag = userLogin;
						listViewUsers.Items.Add(lvi);

						userHashtable["LVI"] = lvi;

						UpdateUser(userLogin);

						textBoxLog.AppendText(DateTime.Now.ToString("yyyy/MM/dd HH:mm:ss.fff") + " '" + userLogin + "' has joined the game.\r\n");

                        PlayerCount++;
					}
					break;
				case "TrackMania.PlayerDisconnect":
					{
						string userLogin = e.Response.Params[0] as string;
						Hashtable userHashtable = users[userLogin] as Hashtable;
						if (userHashtable != null)
						{
							ListViewItem lvi = userHashtable["LVI"] as ListViewItem;
							listViewUsers.Items.Remove(lvi);
							users.Remove(userLogin);
                        }

						textBoxLog.AppendText(DateTime.Now.ToString("yyyy/MM/dd HH:mm:ss.fff") + " '" + userLogin + "' has left the game.\r\n");

                        PlayerCount--;
                    }
					break;
				case "TrackMania.PlayerInfoChanged":
					{
						Hashtable userHashtableNew = e.Response.Params[0] as Hashtable;
						string userLogin = userHashtableNew["Login"] as string;
						Hashtable userHashtable = users[userLogin] as Hashtable;
						if (userHashtable == null)
							users[userLogin] = userHashtable = new Hashtable();

						foreach (string key in userHashtableNew.Keys)
							userHashtable[key] = userHashtableNew[key];

						UpdateUser(userLogin);
					}
					break;
                //TrackMania.PlayerCheckpoint
                //TrackMania.EndRound
                //TrackMania.EndRace
				//TrackMania.StatusChanged
				//TrackMania.ChallengeListModified
				//TrackMania.BeginRace
				//TrackMania.BeginRound
				//TrackMania.PlayerFinish
			}
		}

		private void UpdateUser(string userLogin)
		{
			Hashtable userHashtable = users[userLogin] as Hashtable;
			if (userHashtable != null)
			{
				ListViewItem lvi = userHashtable["LVI"] as ListViewItem;
				if (lvi != null)
				{
                    if (userHashtable.ContainsKey("NickName"))
                        lvi.SubItems[0].Text = userHashtable["NickName"] as string;
                    if (userHashtable.ContainsKey("PlayerId"))
                        lvi.SubItems[2].Text = ((int)userHashtable["PlayerId"]).ToString();
				}
			}
		}

        private void StartAutoStart()
        {
            try
            {
                Connect();
            }
            catch
            {
                timerAutoStart.Enabled = true;
            }
        }

        private void StopAutostart()
        {
            timerAutoStart.Enabled = false;
        }

		private void buttonConnect_Click(object sender, EventArgs e)
		{
            try
            {
			    Connect();
            }
            catch (Exception ex)
            {
                MessageBox.Show("Cannot connect\r\n" + ex.Message);
            }
		}

		private void buttonDisconnect_Click(object sender, EventArgs e)
		{
			Disconnect();
		}

        private void checkBoxAutoConnect_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxAutostart.Checked)
                StartAutoStart();
            else
                StopAutostart();
        }

        void xmlRpcClient_EventOnDisconnectCallback(object o)
		{
			if (InvokeRequired) // Go into the GUI thread
			{
				Invoke(new OnDisconnectHandler(xmlRpcClient_EventOnDisconnectCallback), new object[] { o });
				return;
			}

			textBoxLog.AppendText(DateTime.Now.ToString("yyyy/MM/dd HH:mm:ss.fff") + " The server has been shutdown.\r\n");

			Disconnect();

            if (checkBoxAutostart.Checked)
            {
                timerAutoStart.Enabled = true;
            }
		}

        private void FormTMChecker_Resize(object sender, EventArgs e)
        {
            if (WindowState == FormWindowState.Minimized)
                Hide();
        }

        private void notifyIcon_DoubleClick(object sender, EventArgs e)
        {
            Show();
            WindowState = FormWindowState.Normal;
        }

        private void FormTMChecker_FormClosed(object sender, FormClosedEventArgs e)
        {
            Disconnect();

            Properties.Settings.Default.Save();
        }

        private void timerAutoStart_Tick(object sender, EventArgs e)
        {
            try
            {
                Connect();

                timerAutoStart.Enabled = false;
            }
            catch { }
        }

        private void JoinServer()
        {
            try
            {
                ProcessStartInfo startInfo = new ProcessStartInfo(textBoxTrackManiaClientExecutable.Text);
                startInfo.Arguments = "/lan /join=" + textBoxHost.Text.Trim();
                startInfo.WorkingDirectory = Path.GetDirectoryName(textBoxTrackManiaClientExecutable.Text);
                Process.Start(startInfo);
            }
            catch
            {
                MessageBox.Show("Cannot start the TrackMania client");
            }
        }

        private void buttonBrowseTrackManiaClientExecutable_Click(object sender, EventArgs e)
        {
            openFileDialog.FileName = textBoxTrackManiaClientExecutable.Text;
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                textBoxTrackManiaClientExecutable.Text = openFileDialog.FileName;
            }
        }

        private void notifyIcon_BalloonTipClicked(object sender, EventArgs e)
        {
            JoinServer();
        }

        private void joinTheServerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            JoinServer();
        }

        private void contextMenuStripSystray_Opening(object sender, CancelEventArgs e)
        {
            connectToolStripMenuItem.Enabled = buttonStart.Enabled;
            disconnectToolStripMenuItem.Enabled = buttonStop.Enabled;
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Show();
            Focus();
            Activate();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void connectToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Connect();
        }

        private void disconnectToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Disconnect();
        }

        private void buttonJoinServer_Click(object sender, EventArgs e)
        {
            JoinServer();
        }

        private void buttonAbout_Click(object sender, EventArgs e)
        {
            new AboutForm().ShowDialog();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new AboutForm().ShowDialog();
        }
	}
}
